;-----------------------------------------------------------
; title		: extension pour avr
; files		: ext_avr.asm, 2313def.inc
; revision	: 1.0
; last update	: 30/12/2001
; target	: at90s2313-10pc
; crystal	: 7.3728mhz
; author	: yann leidwanger
; email		: yann.leidwanger@libertysurf.fr
;-----------------------------------------------------------

.include "2313def.inc"

;-----------------------------------------------------------
; definitions & equivalences
;-----------------------------------------------------------

.def	temp	= r16		;mcu
.def	temp2	= r17
.def	canh	= r18		;data h can
.def	canl	= r19		;data l can
.def	cpt	= r20		;compteur
.def	car	= r21		;car uart

.equ	dclk	= pb0		;data clock can
.equ	cs	= pb1		;chip select can
.equ	din	= pb2		;data in can
.equ	dout	= pb3		;data out can

;-----------------------------------------------------------
; config can :
; no power-down
; 12 bits
; 4 entres single-ended
; 24 pulses clock par conversion
;-----------------------------------------------------------
.equ	ch0	= $9f		;channel 0 can
.equ	ch1	= $df		;channel 1 can
.equ	ch2	= $af		;channel 2 can
.equ	ch3	= $ef		;channel 3 can

;-----------------------------------------------------------
; code segment
;-----------------------------------------------------------

.cseg
.org	$0000
	rjmp	reset		;vecteur reset
	rjmp	no_isr		;vecteur int0
	rjmp	no_isr		;int1
	rjmp	no_isr		;t1 capture
	rjmp	no_isr		;t1 compare
	rjmp	no_isr		;t1 overflow
	rjmp	no_isr		;t0 overflow
	rjmp	rx_isr		;vecteur rx uart
	rjmp	no_isr		;udr uart
	rjmp	no_isr		;tx uart
	rjmp	no_isr		;comparator
no_isr:
	reti

;-----------------------------------------------------------
; initialisation systeme
;-----------------------------------------------------------

init_syst:
	clr	car		;acquitte car recu
	ldi	temp,$07
	out	ddrb,temp	;pb0  pb2 sorties
	ldi	temp,$02
	out	portb,temp	;init entrees can
	ldi	temp,$17	;19200 bauds
	out	ubrr,temp	;config vitesse uart
	ldi	temp,$98
	out	ucr,temp	;config + enable uart
	sei			;enable int
	ret

;-----------------------------------------------------------
; it reception uart ; data dans car
;-----------------------------------------------------------

rx_isr:
	in	car,udr		;lit car recu
	reti

;-----------------------------------------------------------
; emission uart ; data dans temp
;-----------------------------------------------------------

uart_emit:
	sbis	usr,udre	;udr empty ?
	rjmp	uart_emit
	out	udr,temp	;envoie data
	ret

;-----------------------------------------------------------
; binary to hexa (ascii) -> uart, data dans temp
;-----------------------------------------------------------

b2h_byte:				;conversion octet
	push	temp
	swap	temp
	rcall	b2h_nibble
	pop	temp
b2h_nibble:				;conversion quartet
	andi	temp,$0f
	ldi	temp2,$f6
	add	temp,temp2
	brcc	b2h_nibble2
	ldi	temp2,$07
	add	temp,temp2
b2h_nibble2:
	ldi	temp2,$3a
	add	temp,temp2
	rcall	uart_emit
	ret

;-----------------------------------------------------------
; lecture can, channel dans temp, resultat dans canh et canl
;-----------------------------------------------------------

lect_can:
	cbi	portb,cs	;enable can

	ldi	cpt,$08		;init compteur bits
loop_1:
	rol	temp		;bit dans carry
	brcs	out_1		;teste carry
out_0:
	cbi	portb,din	;d = 0
	sbi	portb,dclk
	cbi	portb,dclk	;pulse clock
	dec	cpt
	brne	loop_1		;fin de transmission ?
	rjmp	end_1
out_1:
	sbi	portb,din	;d = 1
	sbi	portb,dclk
	cbi	portb,dclk	;pulse clock
	dec	cpt
	brne	loop_1		;fin de transmission ?
end_1:

	ldi	cpt,$08		;init compteur bits
loop_2:
	sbi	portb,dclk	;front montant
	sbis	pinb,dout	;teste etat dout can
	rjmp	in1_0
in1_1:
	sec			;carry = 1
	rol	temp		;ecriture bit
	cbi	portb,dclk	;front descendant
	dec	cpt
	brne	loop_2		;fin de lecture premier octet ?
	rjmp	end_2
in1_0:
	clc			;carry = 0
	rol	temp		;ecriture bit
	cbi	portb,dclk	;front descendant
	dec	cpt
	brne	loop_2		;fin de lecture premier octet ?
end_2:
	mov	canh,temp	;sauvegarde data h can

	ldi	cpt,$08		;init compteur bits
loop_3:
	sbi	portb,dclk	;front montant
	sbis	pinb,dout	;teste etat dout can
	rjmp	in2_0
in2_1:
	sec			;carry = 1
	rol	temp		;ecriture bit
	cbi	portb,dclk	;front descendant
	dec	cpt
	brne	loop_3		;fin de lecture premier octet ?
	rjmp	end_3
in2_0:
	clc			;carry = 0
	rol	temp		;ecriture bit
	cbi	portb,dclk	;front descendant
	dec	cpt
	brne	loop_3		;fin de lecture premier octet ?
end_3:

	rol	temp		;mise en forme de la donnee can
	andi	temp,$f0
	mov	canl,temp
	rol	canh

	sbi	portb,cs	;disable can
	ret

;-----------------------------------------------------------
; programme principal
;-----------------------------------------------------------

reset:
	ldi	temp,low(ramend)
	out	spl,temp	;init sp
	rcall	init_syst	;init systeme
loop:
	ldi	temp,ch0	;channel 0
	rcall	lect_can
	mov	temp,canh
	rcall	b2h_byte
	mov	temp,canl
	rcall	b2h_byte
	ldi	temp,' '
	rcall	uart_emit

	ldi	temp,ch1	;channel 1
	rcall	lect_can
	mov	temp,canh
	rcall	b2h_byte
	mov	temp,canl
	rcall	b2h_byte
	ldi	temp,' '
	rcall	uart_emit

	ldi	temp,ch2	;channel 2
	rcall	lect_can
	mov	temp,canh
	rcall	b2h_byte
	mov	temp,canl
	rcall	b2h_byte
	ldi	temp,' '
	rcall	uart_emit

	ldi	temp,ch3	;channel 3
	rcall	lect_can
	mov	temp,canh
	rcall	b2h_byte
	mov	temp,canl
	rcall	b2h_byte
	ldi	temp,' '
	rcall	uart_emit

	rjmp	loop

;-----------------------------------------------------------
; fin
;-----------------------------------------------------------

